/*****************************************************************
Autor: 			Thomas B.
Beschreibung: 	LCD-Anzeige
Version:		1.0

bekannte Bugs:	keine

Verbesserungsmglichkeit: 	

*****************************************************************/
#include "../Libs/lcd.h"

/****************************************************************
Enable Routine - Enable Pin Toggeln
****************************************************************/
void lcd_toggle_e(void)
{
	LCD_PORT |= (1 << LCD_EN); 		// Enable auf "1"	
	_delay_us(100);	//gehen auch theoretisch weniger...
	LCD_PORT &= ~((1 << LCD_EN));		// Enable auf "0"
	_delay_us(2);//_delay_ms(5);	
}

/********************************************************************
Funktion berprft das Busy-flag des Displays. Solange Busyflag (bit7)
gleich 1 ist, ist das Display beschftigt.
*********************************************************************/
void lcd_w84busy(void)
{
	int busy = 1;
	int data;
	
	LCD_PORT |= (1 << LCD_RW); 		 /* RW=1  read mode      */

	//Port als Input
	LCD_DIR &= ~((1<<DIR_D4) | (1<<DIR_D5) | (1<<DIR_D6) | (1<<DIR_D7));//Pin 1-4 werden auf Eingang gesetzt (==0)
	LCD_PORT |= ((1<<LCD_D4) | (1<<LCD_D5) | (1<<LCD_D6) | (1<<LCD_D7)); // alle pins auf 1 setzen	
	
	while (busy==1)
	{
		/* read high nibble first */
		LCD_PORT |= (1 << LCD_EN); 		// Enable auf "1"	
		_delay_us(1); 
		data = 0;

if (bit_is_clear (LCD_PIN,LCD_PIN_D7)!=0 ) busy = 0;
 /* Die Funktion bit_is_clear prft, ob ein Bit gelscht ist. Wenn das Bit gelscht ist, also auf 0 ist, 
	wird ein Wert ungleich 0 zurckgegeben. 
	bit_is_set entspricht z.B. (Registername & (1 << Bitnummer)). Das Ergebnis
	ist <>0 ("wahr") wenn das Bit gesetzt und 0 ("falsch") wenn es nicht gesetzt ist
 */
		LCD_PORT &= ~((1 << LCD_EN));		// Enable auf "0"
		_delay_us(1);
	
		/* read low nibble */    
		LCD_PORT |= (1 << LCD_EN); 		// Enable auf "1"	
		_delay_us(1);
	
		LCD_PORT &= ~((1 << LCD_EN));		// Enable auf "0"
		_delay_us(1);
	}
	
	LCD_PORT &= ~(1 << LCD_RW); 		 /* RW=0  write mode      */

	//Port als Ausgang
	LCD_DIR |= (1<<DIR_D4) | (1<<DIR_D5) | (1<<DIR_D6) | (1<<DIR_D7);//Pin 1-4 werden auf Ausgang gesetzt (==1)
	LCD_PORT &= ~((1<<LCD_D4) | (1<<LCD_D5) | (1<<LCD_D6) | (1<<LCD_D7) | (1<<LCD_RS) | (1<<LCD_RW) | (1<<LCD_EN)); // alle pins auf 0 setzen	
}

/********************************************************************
Funktion bergibt ein Zeichen an eine bestimmte Position des Displays
char data				-> zu bergebendes Zeichen
unsigned char zeile		-> zeilenangabe: 1,2,3..
unsigned char spalte	-> Spaltenangabe: 1 - x
*********************************************************************/
void lcd_putc_pos(char data,unsigned char zeile, unsigned char spalte)
{
	//Display-position einstellen
	lcd_pos(zeile,spalte);
	//Zeichen senden
	lcd_putc(DATEN,data);
}

/********************************************************************
Funktion bergibt eine Zeichenkette an eine bestimmte Position des Displays
char zeile[LCD_CHARS]	-> zu bergebendes Zeichen
unsigned char zeile		-> zeilenangabe: 1,2,3..
unsigned char spalte	-> Spaltenangabe: 1 - x
*********************************************************************/
void lcd_puts_pos(const char *s,unsigned char zeile, unsigned char spalte)
{
	//Display-position einstellen
	lcd_pos(zeile,spalte);
	//Zeichen senden
	while (*s) 		// == while (*s != '\0')
		lcd_putc(DATEN,*s++);
}


/****************************************************************
Funktion bergibt ein Zeichen an das Display
****************************************************************/
void lcd_putc(unsigned char data_type, unsigned char data_byte)
{
	lcd_w84busy();		
	
	int byte = data_byte, data_bits;
	
	data_bits = 0x00;				// RW (Bit 5) == "0" (RW -> 1==Read mode, 0==Write mode)
	
	if(data_type == DATEN)
		data_bits |= 0x10;			// == RS auf 1 setzen (RS -> 0==BEFEHL, 1==DATEN)
	else
		data_bits|=0x0;				// == RS auf 0 setzen (RS -> 0==BEFEHL, 1==DATEN)


	//senden des higher NIbbles
	LCD_PORT = (data_bits|((byte>>4)&0x0F));	
	lcd_toggle_e();					//Enable toggeln (datenbernahme)

	//senden des lower nibbles
	LCD_PORT = (data_bits|(byte&0x0F));
	lcd_toggle_e();					//Enable toggeln (datenbernahme)

	lcd_w84busy();			
}


/****************************************************************
Funktion bergibt zeichenweise die Daten an die bertragungsfunktion
zeile[LCD_CHARS] 	== zu bertragende Zeichenkette
laenge 			 	== Anzahl der zu bertragenden Zeichen
****************************************************************/
void lcd_puts(const char *s)
{
    while (*s) 	// == while (*s != '\0')
      lcd_putc(DATEN,*s++);
}

/****************************************************************
lscht LCD und setzt cursor auf position 1
****************************************************************/
void lcd_erase(void)
{
	lcd_putc(BEFEHL,0x01);
}

/****************************************************************
Initialisierung des LCDs
****************************************************************/
void lcd_init(void)
{
	LCD_DIR |= (1<<DIR_D4) | (1<<DIR_D5) | (1<<DIR_D6) | (1<<DIR_D7) | (1<<DIR_RS) | (1<<DIR_RW) | (1<<DIR_EN);//Pin 1-4 werden auf Ausgang gesetzt (==1)
	
	//Setzen der Pins
	LCD_PORT &= ~((1<<LCD_D4) | (1<<LCD_D5) | (1<<LCD_D6) | (1<<LCD_D7) | (1<<LCD_RS) | (1<<LCD_RW) | (1<<LCD_EN)); // alle pins auf 0 setzen

	//LCD_PORT &= ~((1<<LCD_D4) | (1<<LCD_D5) | (1<<LCD_D6) | (1<<LCD_D7) | (1<<LCD_RS)); // == auf 0 setzen
	_delay_ms(20);
	
	LCD_PORT = 0x03;
	lcd_toggle_e();		//Enable toggeln (datenbernahme)
	_delay_ms(10); 
	
	LCD_PORT = 0x03;
	lcd_toggle_e();		//Enable toggeln (datenbernahme)
	_delay_ms(3);//(5); 
	
	LCD_PORT = 0x03;
	lcd_toggle_e();		//Enable toggeln (datenbernahme)
	_delay_ms(3);//(5); 

	LCD_PORT = 0x02;
	lcd_toggle_e();		//Enable toggeln (datenbernahme)
	_delay_ms(3);//(5); 
	
#if (LCD_LINES == 2)
	lcd_putc(BEFEHL,0x28);	//	Function Set (RE ==1 --> nur fr DIP204!)
	_delay_ms(3);//(5); 
	
#elif (LCD_LINES == 4)
	lcd_putc(BEFEHL,0x24);	//	Function Set (RE ==1 --> nur fr DIP204!)
	_delay_ms(3);//(5); 
	lcd_putc(BEFEHL,0x09);	//	extended Function Set (4 zeilen)
	_delay_ms(3);//(5); 
	lcd_putc(BEFEHL,0x20);	//	Function Set (RE == 0 --> nur fr DIP204!)
	_delay_ms(3);//(5); 
#else
	#error LCD_Lines nicht 2 oder 4
#endif

	lcd_putc(BEFEHL,0x08);	//0x08	Display off
	_delay_ms(3);//(5); 
	lcd_putc(BEFEHL,0x01);	//0x01  Display clear
	_delay_ms(3);//(5);  
	lcd_putc(BEFEHL,0x06);	//0x06	Display entry mode
	_delay_ms(3);//(5); 

	//ende init
	
	lcd_putc(BEFEHL,0x0F);//0x0F --> Display on
	_delay_ms(3);//(5); 	
	lcd_putc(BEFEHL,0x01);//0x02 -> Display clear	
	_delay_ms(3);//(5); 	
	lcd_putc(BEFEHL,0x02);//0x02 -> return home	
	_delay_ms(3);//(5); 	
}


//****************************************************
// Springt auf Zeile x und Position y der LCD-Anzeige
// Gltig bis 4 Zeilen bei 4x20.
// 4x24 oder 4x40 ??
// unsigned char zeile 		--> Zeilenangabe: 1,2,3...
// unsigned char position 	--> Spaltenangabe, 1-20 (40..)
//****************************************************
void lcd_pos(unsigned char zeile,unsigned char spalte)
{
 unsigned char buff;

// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir  0.
// Zeile 1 -> 0x80...0x93 (bei 20 Zeichen)
// Zeile 2 -> 0x80+0x20 = 0xA0...0xB3 (Display adress set + Adresse)
// Zeile 3 -> 0x80+0x40 = 0xC0...0xD3
// Zeile 4 -> 0x80+0x60 = 0xE0...0xF3

#if (LCD_LINES == 2)
 switch(zeile)
  {
   case 1: buff=0x80+spalte; break;//1
   case 2: buff=0xC0+spalte; break;//3
   case 3: buff=0xA0+spalte; break;//2
   case 4: buff=0xE0+spalte; break;//4
   default : buff=0x80; break; 				//Ungltige Position. Neue => 0,0
  }

#elif (LCD_LINES == 4)
 switch(zeile)
  {
   case 1: buff=0x80+spalte; break;//1
   case 3: buff=0xC0+spalte; break;//3
   case 2: buff=0xA0/*0x94*/+spalte; break;//2
   case 4: buff=0xE0/*0xD4*/+spalte; break;//4
   default : buff=0x80; break; 				//Ungltige Position. Neue => 0,0
  }
#else
	#error LCD_Lines nicht 2 oder 4
#endif

 lcd_putc(BEFEHL,buff);
}


/****************************************************************
Funktion passt kleinere Zeichenketten notfalls an die 20 Zeichen 
des displays an indem die Funktion Leerzeichen an den bergebenen 
String anhngt
str 	== anzupassender Char
laenge	== Laenge des Strings vorm Anpassen
groesse	== gibt an wie lang der string werden soll (wieviele leerzeichen angefgt werden sollen)
****************************************************************/
void zfanpassen(char *str, int laenge, int groesse)
{
	int j;
	
	//Leerzeichen sollen nur angefgt werden, wenn der bergebene string weniger als 20 
	//Zeichen hat
	if((laenge+1)<groesse)
	{
		j=(groesse-1)-laenge;
	
		for(laenge=0;laenge<=j;laenge++)
			str[(groesse-1)-j+laenge] = ' ';
		
		str[groesse] = '\0';
	}
}


/****************************************************************
Konvertiert int to char:
Zahl	= die in char umzuwandelnde int-Gre
str		= char in dem "zahl" stehen soll
laenge	= bergibt die Lnge von str (fr zfanpassen)
komma 	= Komma einfgen ja/nein?
****************************************************************/
void inttostring(int zahl, char *str,int *laenge, int komma)
{
	int temp_zahl;

	temp_zahl = zahl;
	
	for((*laenge)=0,temp_zahl; ;(*laenge)++,temp_zahl=(int)(temp_zahl/10))
		if(temp_zahl==0)
			break;

	itoa (zahl, str, 10); //wandelt eine int Zahl in einen string um -- last "10" specifies that no. is in decimal system.
	
	//Zaheler wird zurckgegeben, damit im Hauptprogramm die Lnge des Strings bekannt ist 
	//(Zeichenkette fr LCD an 20 Zeichen anpassen)
	if(zahl<0)
		(*laenge)++;
		
	if(zahl == 0)
	{
		str[0] = '0';
		str[1] = '0';
		str[2] = '0';
		str[3] = '0';
		str[4] = '0';
		
		*laenge = 4;
	}

	if(komma == KOMMA_POS)
	{
		temp_zahl = (*laenge);
		
		str[temp_zahl] = str[temp_zahl-1];
		temp_zahl--;
		str[temp_zahl] = str[temp_zahl-1];
		temp_zahl--;
		str[temp_zahl] = str[temp_zahl-1];
		temp_zahl--;
		str[temp_zahl] = '.';
		
		(*laenge)++;
	}
}

